//+------------------------------------------------------------------+
//|                                 Close-all-orders-when-profit.mq4 |
//|                        Copyright 2024, tradesmartfxtools.online  |
//|                                https://tradesmartfxtools.online  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, tradesmartfxtools.online"
#property link      "https://tradesmartfxtools.online/"
#property version   "1.03"
#property strict

//+------------------------------------------------------------------+
//|  Global Varibles                                                 |
//+------------------------------------------------------------------+
input double OverallBuyProfit = 0.2; // OverallBuyProfit (combined for all buy orders).
input double OverallSellProfit = 0.2; // OverallSellProfit (combined for all sell orders).

string labelName = "tradesmartfxtools.online";
string labelText = "EA by tradesmartfxtools.online";
string buyProfitLabelName = "OverallBuyProfitLabel";
string sellProfitLabelName = "OverallSellProfitLabel";
int labelFontSize = 18;
color labelColor = Yellow;
color profitLabelColor = Lime;
color lossLabelColor = Red;
int spaceFromBottom = 50;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void createOrUpdateLabels(double buyProfit, double sellProfit)
  {
// Create or update the trademark label
   if(ObjectFind(0, labelName) == -1)
     {
      ObjectCreate(0, labelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, labelName, OBJPROP_CORNER, CORNER_LEFT_LOWER);
   ObjectSetInteger(0, labelName, OBJPROP_XDISTANCE, 10);
   ObjectSetInteger(0, labelName, OBJPROP_YDISTANCE, spaceFromBottom);
   ObjectSetInteger(0, labelName, OBJPROP_COLOR, labelColor);
   ObjectSetInteger(0, labelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, labelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, labelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, labelName, OBJPROP_TEXT, labelText);

// Create or update the buy profit label
   string buyProfitText = "Overall Buy Profit: " + DoubleToString(buyProfit, 2);
   if(ObjectFind(0, buyProfitLabelName) == -1)
     {
      ObjectCreate(0, buyProfitLabelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_XDISTANCE, 20);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_YDISTANCE, spaceFromBottom - 6); // Adjusted Y position
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_COLOR, profitLabelColor);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, buyProfitLabelName, OBJPROP_TEXT, buyProfitText);

// Create or update the sell profit label
   string sellProfitText = "Overall Sell Profit: " + DoubleToString(sellProfit, 2);
   if(ObjectFind(0, sellProfitLabelName) == -1)
     {
      ObjectCreate(0, sellProfitLabelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_XDISTANCE, 20);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_YDISTANCE, spaceFromBottom - 36); // Adjusted Y position
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_COLOR, lossLabelColor);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, sellProfitLabelName, OBJPROP_TEXT, sellProfitText);
  }

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   createOrUpdateLabels(0.0, 0.0); // Initialize labels with 0 profit

   return(INIT_SUCCEEDED);
  }

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  }

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   double totalBuyProfit = 0.0;
   double totalSellProfit = 0.0;

// Calculate total buy and sell profits
   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol)
        {
         if(OrderType() == OP_BUY)
            totalBuyProfit += OrderProfit();
         else
            if(OrderType() == OP_SELL)
               totalSellProfit += OrderProfit();
        }
     }

// Update all labels with current profit values
   createOrUpdateLabels(totalBuyProfit, totalSellProfit);

// Close orders if profit thresholds are met
   CloseAllBuyOrdersIfProfitAbove(OverallBuyProfit);
   CloseAllSellOrdersIfProfitAbove(OverallSellProfit);
  }

//+------------------------------------------------------------------+
//|    Close All Buy Orders If Profit Above                          |
//+------------------------------------------------------------------+
void CloseAllBuyOrdersIfProfitAbove(double threshold)
  {
   double totalBuyProfit = 0.0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
        {
         totalBuyProfit += OrderProfit();
        }
     }

   if(totalBuyProfit >= threshold)
     {
      for(int i = OrdersTotal() - 1; i >= 0; i--)
        {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
           {
            if(OrderClose(OrderTicket(), OrderLots(), Bid, 2, Blue) == true)
              {
               Print("Buy Order closed successfully. Ticket: ", OrderTicket());
              }
            else
              {
               Print("Error closing Buy Order. Error code: ", GetLastError());
              }
           }
        }
     }
  }

//+------------------------------------------------------------------+
//|    Close All Sell Orders If Profit Above                         |
//+------------------------------------------------------------------+
void CloseAllSellOrdersIfProfitAbove(double threshold)
  {
   double totalSellProfit = 0.0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
        {
         totalSellProfit += OrderProfit();
        }
     }

   if(totalSellProfit >= threshold)
     {
      for(int i = OrdersTotal() - 1; i >= 0; i--)
        {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
           {
            if(OrderClose(OrderTicket(), OrderLots(), Ask, 2, Red) == true)
              {
               Print("Sell Order closed successfully. Ticket: ", OrderTicket());
              }
            else
              {
               Print("Error closing Sell Order. Error code: ", GetLastError());
              }
           }
        }
     }
  }
//+------------------------------------------------------------------+
